//=============================================================================
// Copyright (c) 2016-present Allan CORNET (Nelson)
//=============================================================================
// This file is part of the Nelson.
//=============================================================================
// LICENCE_BLOCK_BEGIN
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// Alternatively, you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program. If not, see <http://www.gnu.org/licenses/>.
// LICENCE_BLOCK_END
//=============================================================================
#include "actxserverBuiltin.hpp"
#include "ActiveXServer.hpp"
#include "Error.hpp"
//=============================================================================
using namespace Nelson;
//=============================================================================
ArrayOfVector
Nelson::ComEngineGateway::actxserverBuiltin(int nLhs, const ArrayOfVector& argIn)
{
    ArrayOfVector retval;
#ifdef _MSC_VER
    nargoutcheck(nLhs, 0, 1);
    std::wstring progid;
    std::wstring machine;
    switch (argIn.size()) {
    case 1: {
        progid = argIn[0].getContentAsWideString();
    } break;
    case 3: {
        std::wstring type = argIn[2].getContentAsWideString();
        if (!(type == L"machine")) {
            Error(_W("'machine' value expected."));
        }
        progid = argIn[0].getContentAsWideString();
    } break;
    case 2:
    default: {
        Error(ERROR_WRONG_NUMBERS_INPUT_ARGS);
    } break;
    }
    ComHandleObject* comhandle = ActiveXServer(progid, machine);
    retval << ArrayOf::handleConstructor(comhandle);
#else
    Error(_W("Not implemented on this platform."));
#endif
    return retval;
}
//=============================================================================
